function computeGeometry "Compute geometry" 
  
annotation (
 Documentation(
        info="<HTML>
This function computes the <b>theStartVec3D</b> and the <b>theEndVec3D</b> of a free span.
<br><br>
<b>belongs to:</b>
Span.mo
<br><br>
</HTML>"));
  
 input Modelica.SIunits.Position inFirstCenter3D[3] 
    "3-dimensional position vector of the first pulley";
 input Modelica.SIunits.Diameter inFirstDia "diameter of the first pulley";
 input Integer inFirstRot "rotation of the first pulley";
 input Modelica.SIunits.Position inSecondCenter3D[3] 
    "3-dimensional position vector of the first pulley";
 input Modelica.SIunits.Diameter inSecondDia "diameter of the second pulley";
 input Integer inSecondRot "rotation of the second pulley";
  
 // start vector
 output Modelica.SIunits.Position theStartVec3D[3] 
    "3-dimensional vector of the start position of the span";
  
 // end vector
 output Modelica.SIunits.Position theEndVec3D[3] 
    "3-dimensional vector of the end position of the span";
  
protected 
 Modelica.SIunits.Position inFirstCenter[2] = {inFirstCenter3D[1], inFirstCenter3D[2]};
 Modelica.SIunits.Position inSecondCenter[2] = {inSecondCenter3D[1], inSecondCenter3D[2]};
 Modelica.SIunits.Length theCrossDist;
 Real thePerp;
 Real theLong;
 Real theSqrtArg;
  
 // radius
 Modelica.SIunits.Radius theFromRad = inFirstDia / 2.0;
 Modelica.SIunits.Radius theToRad = inSecondDia / 2.0;
  
 // diameter ratio (to/from)
 Real theRatio = theToRad / theFromRad;
  
 // compute distance vector
 Modelica.SIunits.Position theDistanceVec[2] = inSecondCenter - inFirstCenter;
  
 // compute unit vector(s)
 Modelica.SIunits.Position theUnitVec[2] = theDistanceVec / absVec(theDistanceVec);
  
 // this is already normalized (above)...
 Modelica.SIunits.Position thePerpUnitVec[2] = {-theUnitVec[2], theUnitVec[1]};
  
 // length of distance vector
 Modelica.SIunits.Length theDistVecLength = absVec(theDistanceVec);
  
 // start vector
 Modelica.SIunits.Position theStartVec[2];
  
 // start vector
 Modelica.SIunits.Position theEndVec[2];
  
 // direction
 Modelica.SIunits.Position theDirection;
  
algorithm 
 assert(theDistVecLength > (theFromRad + theToRad),"failed to find tangent");
  
 if (inFirstRot <> inSecondRot) then
    
    // similarity of triangles:
    // distance to crossing between tangent and distance vector
    theCrossDist := theDistVecLength*theFromRad/(theFromRad + theToRad);
    
    // components of the vector between the center and the
    // tangent on the "from" circle (in distance vector coordinates).
    
    // sqrt(theSqrtArg) is the distance from tangent point
    // on circle to cross dist point on distance line.
    theSqrtArg := theCrossDist*theCrossDist - theFromRad*theFromRad;
    
    // similarity of triangles: 
    // (lengths from fromCenter to fromTangentPoint)
    
    // this is the perpendicular component
    thePerp := theFromRad/theCrossDist*sqrt(theSqrtArg);
    
    // sqrt(theSqrtArg) is the longitudinal component
    theSqrtArg := theFromRad*theFromRad - thePerp*thePerp;
    
    // longitudinal component
    theLong :=sqrt(theSqrtArg);
    
    theStartVec :=(theLong*theUnitVec) + (thePerp * inFirstRot * thePerpUnitVec);
    
    // components of the vector between the center and the
    // tangent on the "to" circle (in distance vector coordinates)
    thePerp :=thePerp*theRatio;
    theLong :=theLong*theRatio;
    
    theEndVec := (-theLong * theUnitVec) + (thePerp * inSecondRot * thePerpUnitVec);
 else
    
    // if the pulleys have equal size the crossing is at infinity
    if abs(theFromRad - theToRad) > (1.0 / 10^12) then
      
        // distance to crossing between tangent and distance vetor
        theCrossDist := theDistVecLength*theFromRad/abs(theFromRad - theToRad);
      
        // components of the vector between the center and the
        // tangent on the "from" circle (in distance vector coordinates)
        theSqrtArg := theCrossDist*theCrossDist - theFromRad * theFromRad;
      
        thePerp := theFromRad / theCrossDist*sqrt(theSqrtArg);
        theSqrtArg := theFromRad * theFromRad - thePerp * thePerp;
      
        theLong := sqrt(theSqrtArg);
      
    else
        thePerp := theFromRad;
        theLong := 0.0;
    end if;
    
    if (theFromRad > theToRad) then
        theDirection := 1.0;
    else
        theDirection := -1.0;
    end if;
    
    theStartVec := (theDirection * theLong * theUnitVec) + (thePerp * inFirstRot * thePerpUnitVec);
    
    // components of the vector between the center and the
    // tangent on the "to" circle (in distance vector coordinates)
    thePerp :=thePerp*theRatio;
    theLong :=theLong*theRatio;
    
    theEndVec := (theDirection * theLong * theUnitVec) + (thePerp * inSecondRot * thePerpUnitVec);
    
 end if;
  
 // compute unit vectors --- in world coordinates
 theStartVec := theStartVec + inFirstCenter;
 theEndVec := theEndVec + inSecondCenter;
  
 // 3D span geometry
 theStartVec3D := {theStartVec[1],theStartVec[2], 0};
 theEndVec3D := {theEndVec[1],theEndVec[2], 0};
  
end computeGeometry;
