model Span "Span" 
  extends Belts.Interfaces.TwoConnectors;
  replaceable Belts.Visualizers.BeltShapes.FlatBelt beltShape extends 
    Belts.Interfaces.PartialBeltShape "beltShape" annotation(choicesAllMatching= true);
  import Modelica.Constants.pi;
  annotation (preferedView="info",Documentation(info="<HTML>
 <p>
  <IMG SRC=\"../Pics/Span.png\"
</p> 
<p>
This model element is a fixed or free span with transversal degrees of freedom. It computes <b>span 
length</b>, <b>span force</b> and a vector <b>q</b> of size <b>modeShapes</b> for the tranversal span vibrations.<br>
After double clicking on the span element it is possible to select different types of belts in the <b>General</b> tab. In the <b>beltShape</b> 
pull down menu, you can choose between double profile belts, flat belts and v-belts.
 
<br><br><br>
 
<b>Span vibrations </b>
<br>
<p>
Span vibrations represent system-specific effects in belt wrap transmissions. In principle, the traction mechanism can be excited
to produce transversal, longitudinal and torsional vibrations. In comparison with the bending rigidity, however, the rigidity
of the belt in a longitudinal direction or about the torsional axis is very large.
Span vibrations are caused by irregularities in the drive torque.
The appearance of span vibrations can be attributed to several physical effects.
The mechanism of parameter excitation as a result of oscillating strand forces is not
in dispute. In addition, deviations from the ideal geometrical form, for example in eccentric
pulleys and local variations in belt characteristics, represent a source of parameter
excitation. A transport effect also occurs as a result of belt recirculation,
which couples the longitudinal and transversal motion. In practice, it is found
that the span vibration amplitudes have an asymmetrical form.
This effect has not been taken into consideration as yet. <br>
</p> 
 
<b>Mechanical model </b>
<p> 
The modelling of belt transversal vibrations
as string vibrations of the slack
strands is based closely on the description above. By using basic functions for the belt
transversal motions according to the RITZ method in accordance with
</p>
<p align=\"center\">
<IMG SRC=\"../Pics/eq9.jpg\"
</p>
<p>
with a vector w of purely location-dependent basic functions and the vector q of
pure time functions representing degrees of freedom in the overall mechanical model,
a representation is obtained that is transformed into conventional differential
formulae:
</p>
<p align=\"center\">
<IMG SRC=\"../Pics/eq10.jpg\"
</p>
<p>
where
</p>
<p align=\"center\">
<IMG SRC=\"../Pics/eq10a-e.jpg\"
</p>
<p>
Damping proportional to speed was
added here with a matrix based on the
rigidity matrix. The rotation points are assumed
to be stationary. In the calculation
of the longitudinal belt force, the analogy
to calculation in the previous section can be
seen: only changes in length and corresponding
elongation speeds due to transversal
vibrations have been added.
The basic functions used for transversal
vibrations are sine functions for which a
whole-number multiple of half the wavelength
corresponds to the slack length of
the belt strand: wi(x) = sin(i*Pi/l0*x).<br>
As a result, the location integral matrices
can be stated explicitly. For the jth basic
function, this gives the following differential
formula
</p>
 <p align=\"center\">
<IMG SRC=\"../Pics/eq11.jpg\"
</p>
<p>
</p>
<b>Animation </b>
<p>
To accelerate the simulation time it is possible to decrease the number of animation points, by selecting the tab <b>Animation</b> and enter
e.g. two at <b>animationPoints</b>.  
 
</p>
Detailed information about the model can be found <a href=\"..\UsersGuide\Dynamic Simulation of Automobile Accessory Drives.pdf\">
<b>here</b></a>.
</HTML>
"));
  
  annotation (Icon(
    Rectangle(
      extent=[-100,3; 100,-3],
      style(
        color=1,
        rgbcolor={255,0,0},
        fillColor=1,
        rgbfillColor={255,0,0})),
   Text(
      extent=[-120,46; -80,12],
      string="a",
      style(color=1,
      rgbcolor={255,0,0},
      fillColor=1,
      rgbfillColor={255,0,0},
      fillPattern=1)),
    Text(
      extent=[80,46; 120,12],
      string="b",
        style(
          color=1,
          rgbcolor={255,0,0},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1))),
    Diagram);
  
  import Modelica.Math.asin;
  import Modelica.Math.cos;
  
  annotation (Icon(Text(extent=[-120,44; -80,10], string="a", style(color=1, rgbcolor={255,0,0},
    fillColor=1, rgbfillColor={255,0,0}, fillPattern=1)), Text(extent=[80,44; 120,10],
    style(color=3, rgbcolor={0,0,255}, fillColor=3, rgbfillColor={0,0,255}, fillPattern=1),
    string="b")), DymolaStoredErrors);
  
  Modelica.SIunits.Force SpanForce;
  
  parameter Integer modeShapes = 3 
    "number of modes shapes for transversal vibration";
  parameter Modelica.SIunits.Length InitialElongation = Belts.Defaults.InitialElongation 
    "initial elongation for all mode shapes";
  parameter Integer animationPoints = 50 
    "number of points along the spanlength" 
    annotation(Dialog(tab="Animation"));
  parameter Boolean animateSpanForces = true "show span force vector" 
    annotation(Dialog(tab="Animation", group="span forces", enable= showPulley));
  parameter Belts.Types.ForceScalingFactor spanForceScale= Belts.Defaults.spanForceScale 
    "scaling factor for span force vector" 
    annotation(Dialog(tab="Animation", group="span forces", enable=animateSpanForces));
  parameter Modelica.Mechanics.MultiBody.Types.Color spanForceColor = Belts.Defaults.spanForceColor 
    "color of arrow representing the span force vector" 
    annotation(Dialog(tab="Animation", group="span forces", enable=animateSpanForces));
  parameter Modelica.SIunits.Diameter spanForceDiameter = Belts.Defaults.spanForceDiameter 
    "diameter of arrow representing the span force vector" 
    annotation(Dialog(tab="Animation", group="span forces", enable=animateSpanForces));
 Modelica.SIunits.Length q[modeShapes](start={InitialElongation for i in 1:modeShapes}) 
    "transversal vibration";
  
  parameter Belts.Types.BeltStiffness EA= Belts.Defaults.EA 
    "longitudinal belt stiffness" 
    annotation(Dialog(group="belt parameters"));
  
  parameter Belts.Types.BeltMass mr = Belts.Defaults.mr "belt mass per meter" 
    annotation(Dialog(group="belt parameters"));
  
  parameter Belts.Types.SpanForce F0 = Belts.Defaults.F0 "preload" 
    annotation(Dialog(group="belt parameters"));
  
  parameter Belts.Types.BeltDamping d_l = Belts.Defaults.d_l 
    "longitudinal belt damping coefficient" 
    annotation(Dialog(group="belt parameters"));
  
  parameter Belts.Types.BeltDamping d_t = Belts.Defaults.d_t 
    "transversal belt damping coefficient" 
    annotation(Dialog(group="belt parameters"));
  
  parameter Modelica.SIunits.Length h= Belts.Defaults.height "belt height" 
    annotation(Dialog(group="Animation"));
  
  parameter Modelica.SIunits.Length width = Belts.Defaults.width "belt width" 
    annotation(Dialog(group="Animation"));
  
  parameter Modelica.SIunits.Position z = Belts.Defaults.z 
    "z-coordinate of belt drive" 
    annotation(Dialog(group="Animation"));
  
protected 
  outer Modelica.Mechanics.MultiBody.World world;
  
  // span geometry
  Real TotalSpanLength;
  Modelica.SIunits.Length NominalTotalSpanLength;
  Modelica.SIunits.Length CurrentSpanElongation;
  Modelica.SIunits.Length deltaL;
  Modelica.SIunits.Force theDampingForce;
  Modelica.SIunits.Force SpanForceVec[3];
  Modelica.SIunits.Length CurrentSpanLength;
  Modelica.SIunits.Length NominalSpanLength;
  Modelica.SIunits.Position SpanEnd[3];
  Modelica.SIunits.Position SpanStart[3];
  Modelica.SIunits.Position[3] SpanVec;
  Modelica.SIunits.Position[3] Xm;
  // ----------------------------- transversal span vibration
  Real M[modeShapes,modeShapes];
  Real D[modeShapes, modeShapes];
  Real w_w_T[modeShapes, modeShapes];
  Real G[modeShapes, modeShapes];
  
  Modelica.SIunits.Velocity v_q[modeShapes] "transversal velocity";
  Modelica.SIunits.Acceleration a_q[modeShapes] "transversal velocity";
  Real beltSpeed;
  
Belts.Visualizers.Span SpanVis(
      ContactPoint_2={SpanStart[1],SpanStart[2],z},
      ContactPoint_1={SpanEnd[1],SpanEnd[2],z},
      PullyDirection_a=spanConnector_a.direction,
      PullyDirection_b=spanConnector_b.direction,
      h=h,
      b=width,
      shape=beltShape.y,
      numPnts=animationPoints,
      ampl=q) if world.enableAnimation;
  
  // spanForce vectors 
  Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow InSpanForceVector(color = spanForceColor,
    r_head = {SpanForceVec[1] * spanForceScale, SpanForceVec[2] * spanForceScale, 0},
    r = {SpanStart[1], SpanStart[2], z + width * 0.6},
    diameter = spanForceDiameter) if world.enableAnimation and animateSpanForces;
  
  Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow OutSpanForceVector(color = spanForceColor,
    r_head = {-SpanForceVec[1] * spanForceScale, -SpanForceVec[2] * spanForceScale, 0},
    r = {SpanEnd[1], SpanEnd[2], z + width * 0.6},
    diameter = spanForceDiameter) if world.enableAnimation and animateSpanForces;
  
equation 
  // initial values
  when {initial()} then
    NominalSpanLength = CurrentSpanLength;
    NominalTotalSpanLength = CurrentSpanLength;
  end when;
  
  // evaluate belt speed
  beltSpeed = (abs(spanConnector_a.Rotation * spanConnector_a.PulleyDiameter/2) + abs(spanConnector_b.Rotation * spanConnector_b.PulleyDiameter/2))/2;
  
  // evaluate longitudinal belt damping force
  theDampingForce = d_l * (der(spanConnector_a.Rotation) * spanConnector_a.PulleyDiameter/2 *spanConnector_a.direction - der(spanConnector_b.Rotation) * spanConnector_b.PulleyDiameter/2 * spanConnector_b.direction);
  
  // change of span length
  CurrentSpanElongation = TotalSpanLength - NominalTotalSpanLength;
  
  // vector between pulley positions
  Xm = spanConnector_a.PulleyPosition - spanConnector_b.PulleyPosition;
  
  // span vector
  SpanVec = -SpanEnd + SpanStart;
  
  // current span length
  CurrentSpanLength = sqrt(SpanVec[1]^2 + SpanVec[2]^2);
  
  // current span length + parts of span on pulley
  TotalSpanLength = spanConnector_a.DeltaWALength  + CurrentSpanLength + spanConnector_b.DeltaWALength;
  
  // change of total span length  
  deltaL = TotalSpanLength - NominalTotalSpanLength;
  
  // span force vector
  SpanForceVec = SpanVec/CurrentSpanLength * SpanForce;
  
  // span force
  SpanForce = max( 0, theDampingForce + EA  * (q*w_w_T*q + (deltaL + (spanConnector_a.PulleyDiameter/2 * spanConnector_a.direction *spanConnector_a.Rotation - spanConnector_b.PulleyDiameter/2 * spanConnector_b.direction *spanConnector_b.Rotation)) /CurrentSpanLength) + F0);
  (SpanStart, SpanEnd) =  Belts.Functions.computeGeometry(spanConnector_a.PulleyPosition, spanConnector_a.PulleyDiameter, spanConnector_a.direction,
    spanConnector_b.PulleyPosition, spanConnector_b.PulleyDiameter, spanConnector_b.direction);
  
  spanConnector_a.Xm = Xm;
  spanConnector_b.Xm = -Xm;
  spanConnector_a.SpanForce = SpanForceVec;
  spanConnector_b.SpanForce = -SpanForceVec;
  spanConnector_a.ContactPoint =  SpanStart;
  spanConnector_b.ContactPoint =  SpanEnd;
  spanConnector_a.SpanVec2D = {SpanVec[1],SpanVec[2]};
  spanConnector_b.SpanVec2D = {SpanVec[1],SpanVec[2]};
  spanConnector_a.SpanLength = CurrentSpanLength;
  spanConnector_b.SpanLength = CurrentSpanLength;
  
  //  zero mode shapes = no span vibrations
  if modeShapes==0 then
    M=identity(modeShapes);
    D=identity(modeShapes);
    w_w_T=identity(modeShapes);
    G=identity(modeShapes);
    q= {1 for i in 1: modeShapes};
  else
  M = identity(modeShapes)*mr*CurrentSpanLength/2;
  D = {if i==j then (i*pi)^2*d_t/(2*CurrentSpanLength) else 0 for i in 1:modeShapes, j in 1:modeShapes};
  w_w_T = {if i==j then (i*pi)^2/(2*CurrentSpanLength) else 0 for i in 1:modeShapes, j in 1:modeShapes};
  G = {if i<>j then ((1-cos(pi*(i-j)))/(2*(i-j)) + (1-cos(pi*(i+j)))/(2*(i+j))) else 0 for i in 1:modeShapes, j in 1:modeShapes};
    
  M*a_q + (G+D)*v_q + (SpanForce - beltSpeed^2*mr)*w_w_T*q = zeros(modeShapes);
  end if;
  v_q = der(q);
  a_q = der(v_q);
  
end Span;
