model WrapAngle "Wrap angle" 
   replaceable Belts.Visualizers.BeltShapes.FlatBelt beltShape extends 
    Belts.Interfaces.PartialBeltShape "beltShape" annotation(choicesAllMatching= true);
  annotation (Documentation(info="<HTML>
 <p>
  <IMG SRC=\"../Pics/WrapAngle.png\"
</p> 
<p>
This model element computes resulting <b>hubload</b> and <b>torque</b> on the pulley.
After double clicking on the wrap angle element it is possible to select different types of belts in the <b>General</b> tab. In the 
<b>beltShape</b> pull down menu, you can choose between double profile belts, flat belts and v-belts.
</p>
</HTML>
"), Diagram);
  
  annotation (Icon(Polygon(points=[-34,-32; -28,-32; -28,-32; -28,-22; -24.38,
            -11.48; -19.96,-2; -13.96,6.57; -6.57,13.96; 2,19.96; 11.48,24.38;
            21.58,27.09; 32,28; 32,28; 32,34; 32,34; 20.54,33; 9.43,30.02; -1,
            25.16; -10.42,18.56; -18.56,10.42; -25.16,1; -30.02,-9.43; -34,-22;
            -34,-32],
    style(
      color=1,
      rgbcolor={255,0,0},
      fillColor=1,
      rgbfillColor={255,0,0})),
    Text(
      extent=[-70,-34; -28,-68],
      string="b",
        style(
          color=1,
          rgbcolor={255,0,0},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1)),
    Text(
      extent=[32,68; 72,34],
      string="a",
      style(color=1,
      rgbcolor={255,0,0},
      fillColor=1,
      rgbfillColor={255,0,0},
      fillPattern=1))),
    Diagram,
    Coordsys(grid=[2,2], scale=0.1),
    DymolaStoredErrors);
  
  import Belts.Functions.absVec;
  import Belts.Functions.computeWrapAngle;
  import Modelica.Math.acos;
  import Modelica.Math.atan;
  import Modelica.Constants.pi;
  
  parameter Real beltMass = Belts.Defaults.mr "belt mass per length";
  // friction coefficient Mu = Mu* = MuH/(sin(alpha/2))
  parameter Real mu = Belts.Defaults.Mu "friction coefficient" annotation(Dialog(group="belt slip"));
  
  parameter Modelica.SIunits.Length h= Belts.Defaults.height "belt height" 
    annotation(Dialog(group="Animation"));
  
  parameter Modelica.SIunits.Length width = Belts.Defaults.width "belt width" 
    annotation(Dialog(group="Animation"));
  
  parameter Modelica.SIunits.Position z = Belts.Defaults.z 
    "z-coordinate of belt drive" 
    annotation(Dialog(group="Animation"));
  
  Real WrapAngle;
//  Real slipRisk;
  
  Interfaces.SpanConnector_b spanConnector_b "connection to span \"a\"" 
    annotation (extent=[-40,-40; -20,-20]);
  Interfaces.SpanConnector_a spanConnector_a "connection to span \"b\"" 
    annotation (extent=[20,20; 40,40]);
  Interfaces.WrapAngleConnector_b wrapAngleConnector_b "connection to pulley" 
    annotation (extent=[-22,0; -2,20]);
/*
  annotation (Documentation(info="<HTML>
<p>
This is a multiple connector belonging to the advanced model element <b>WrapAngle</b>.
</p>
</HTML>
"));
*/
protected 
  outer Modelica.Mechanics.MultiBody.World world;
  
  Real WALength "length of wrap angle";
  Real NominalWrapAngle "wrap angle in nominal position";
  Real WrapAngleIn;
  Real DeltaWALengthIn;
  Real WrapAngleOut;
  Real DeltaWALengthOut;
  Real DeltaWAIn;
  Real DeltaWAOut;
  Real VecIn[3];
  Real VecOut[3];
  Modelica.SIunits.Force Hubload[3];
  Modelica.SIunits.Torque Torque;
  Real theVecIn[2];
  Real NominalStartVecIn[3];
  Real NominalSpanVec2DIn[2];
  Real NominalStartVecOut[3];
  Real NominalSpanVec2DOut[2];
  
// Visualizer
  
  Belts.Visualizers.WrapAngle WrapAngleVis(
      PulleyPosition = {wrapAngleConnector_b.PulleyPosition[1], wrapAngleConnector_b.PulleyPosition[2], z},
      b = width,
      PullyDirection = wrapAngleConnector_b.direction,
      ri = wrapAngleConnector_b.PulleyDiameter/2,
      shape=beltShape.y,
      h=h,
      startAngle=Modelica.Math.atan2(VecIn[2],VecIn[1]),
      stopAngle = Modelica.Math.atan2(VecIn[2],VecIn[1])- spanConnector_a.direction*WrapAngle) if world.enableAnimation;
  
equation 
  // initial values 
  when {initial()} then
    NominalStartVecIn = spanConnector_a.ContactPoint - spanConnector_a.PulleyPosition;
    NominalSpanVec2DIn = spanConnector_a.SpanVec2D;
    NominalStartVecOut = spanConnector_b.ContactPoint - spanConnector_b.PulleyPosition;
    NominalSpanVec2DOut = spanConnector_b.SpanVec2D;
  end when;
  
  // evaluation of hubload
  Hubload = spanConnector_b.SpanForce + spanConnector_a.SpanForce;
  wrapAngleConnector_b.f = -Hubload;
  
  // evaluation of torque
  Torque = wrapAngleConnector_b.direction * wrapAngleConnector_b.PulleyDiameter/2 * (-absVec(spanConnector_a.SpanForce) + absVec(spanConnector_b.SpanForce));
  
  // length of wrap angle
  WALength =  WrapAngle * wrapAngleConnector_b.PulleyDiameter/2;
  
  // current wrap angle
  DeltaWALengthIn = (WrapAngleIn - NominalWrapAngle) * wrapAngleConnector_b.PulleyDiameter/2;
  DeltaWALengthOut = (WrapAngleOut - NominalWrapAngle) * wrapAngleConnector_b.PulleyDiameter/2;
  DeltaWAIn = WrapAngleIn - NominalWrapAngle;
  DeltaWAOut = WrapAngleOut - NominalWrapAngle;
  
  // vector directed from pulley axis to contact point
  VecIn = spanConnector_a.ContactPoint - spanConnector_a.PulleyPosition;
  VecOut = spanConnector_b.ContactPoint - spanConnector_b.PulleyPosition;
/*  
  //eytelwein
  wrapAngleConnector_b.eytelweinTorque= if  absVec(spanConnector_a.SpanForce)< absVec(spanConnector_b.SpanForce) then 
                     absVec(spanConnector_a.SpanForce)*exp(mu*WrapAngle)*wrapAngleConnector_b.PulleyDiameter/2 else 
                     absVec(spanConnector_b.SpanForce)*exp(mu*WrapAngle)*wrapAngleConnector_b.PulleyDiameter/2;
  
  slipRisk= if (abs(Torque)/(wrapAngleConnector_b.eytelweinTorque+1e-20))>1 then 1 else abs(Torque)/(wrapAngleConnector_b.eytelweinTorque+1e-20);
 */
algorithm 
  // evaluation of wrap angles
  WrapAngle := computeWrapAngle(VecOut,spanConnector_a.SpanVec2D,spanConnector_b.SpanVec2D);
  NominalWrapAngle := computeWrapAngle(NominalStartVecOut,NominalSpanVec2DIn,NominalSpanVec2DOut);
  WrapAngleIn := computeWrapAngle(VecOut,spanConnector_a.SpanVec2D,NominalSpanVec2DOut);
  WrapAngleOut := computeWrapAngle(VecOut,NominalSpanVec2DIn,spanConnector_b.SpanVec2D);
  
  theVecIn := {spanConnector_a.ContactPoint[1] - spanConnector_a.PulleyPosition[1],spanConnector_a.ContactPoint[2] - spanConnector_a.PulleyPosition[2]};
  
equation 
  wrapAngleConnector_b.PulleyPosition = spanConnector_a.PulleyPosition;
  wrapAngleConnector_b.PulleyDiameter = spanConnector_a.PulleyDiameter;
  wrapAngleConnector_b.direction = spanConnector_a.direction;
  wrapAngleConnector_b.Rotation = spanConnector_a.Rotation;
  wrapAngleConnector_b.WrapAngle = spanConnector_a.WrapAngle;
  
  wrapAngleConnector_b.PulleyPosition = spanConnector_b.PulleyPosition;
  wrapAngleConnector_b.PulleyDiameter = spanConnector_b.PulleyDiameter;
  wrapAngleConnector_b.direction = spanConnector_b.direction;
  wrapAngleConnector_b.Rotation = spanConnector_b.Rotation;
  wrapAngleConnector_b.WrapAngle = spanConnector_b.WrapAngle;
  
  spanConnector_a.DeltaWALength = DeltaWALengthIn;
  spanConnector_b.DeltaWALength = DeltaWALengthOut;
  
  width = wrapAngleConnector_b.width;
  z = wrapAngleConnector_b.z;
  
  wrapAngleConnector_b.torque = Torque;
  wrapAngleConnector_b.WrapAngle = WrapAngle;
  
  wrapAngleConnector_b.spanForceVec_a = spanConnector_a.SpanForce;
  wrapAngleConnector_b.spanForceVec_b = spanConnector_b.SpanForce;
  wrapAngleConnector_b.spanLength_a = spanConnector_a.SpanLength;
  wrapAngleConnector_b.spanLength_b = spanConnector_b.SpanLength;
  wrapAngleConnector_b.spanHeight = h;
end WrapAngle;
